#! /bin/bash

passwordHash () {
    password=${1}
    salt=${2}
    encryption=${3}

    if [[ -x /usr/local/openssl/bin/openssl ]]; then
        # When updating /usr/local/openssl/bin/openssl, check that this script / this command still works.
        # @dependency /usr/local/openssl/bin/openssl
        hashes=$(echo ${password} | /usr/local/openssl/bin/openssl passwd -${encryption} -salt ${salt} -stdin)
    else
        hashes=$(echo ${password} | openssl passwd -${encryption} -salt ${salt} -stdin)
    fi

    echo $(substring ${hashes} "$" "3")
}


passwordIsValid () {

    user=${1}
    password=${2}

    encryption=$(secret "encryption" ${user})
    salt=$(secret "salt" ${user})
    salted=$(secret "salted" ${user})
    hash=$(passwordHash ${password} ${salt} ${encryption})

    [ ${salted} = ${hash} ] && echo "true" || echo "false"
}


secret () {
    secret=${1}
    user=${2}

    shadow=$(shadow ${user})

    if [ ${secret} = "encryption" ]; then
        position=1
    elif [ ${secret} = "salt" ]; then
        position=2
    elif [ ${secret} = "salted" ]; then
        position=3
    fi

    echo $(substring ${shadow} "$" ${position})
}


shadow () {
    user=${1}

    shadow=$(cat /etc/shadow | grep ${user})
    shadow=$(substring ${shadow} ":" "1")
    echo ${shadow}
}


substring () {
    string=${1}
    separator=${2}
    position=${3}

    substring=${string//"${separator}"/$'\2'}
    IFS=$'\2' read -a substring <<< "${substring}"
    echo ${substring[${position}]}
}


passwordIsValid ${@}