#!/usr/bin/env bash

COUNTER=0
isLB="False"
while [  $COUNTER -lt 1 ]; do
    pcsd_status=$(systemctl is-active --quiet pcsd && echo Service is running)
    pacemaker_status=$(systemctl is-active --quiet pacemaker && echo Service is running)
    corosync_status=$(systemctl is-active --quiet corosync && echo Service is running)

    if [[ "$pcsd_status" == "Service is running" && "$pacemaker_status" == "Service is running" && "$corosync_status" == "Service is running" ]]; then
        let COUNTER=COUNTER+1
    else
        if [[ -f /home/akkadianuser/load_balancer.config ]]; then
            nodes=$(head -n 1 /home/akkadianuser/load_balancer.config)
            if [[ ! -v "${nodes}" ]]; then
                let COUNTER=COUNTER+1
                isLB="True"
            fi
        fi

    fi
    sleep 1
done

if [[ "$isLB" == "False" ]]; then
    MASTER=$(sudo pcs status | grep RS_VIP_Akkadian | awk -F ' ' '{print $4}')
    NODES=$(sudo pcs status | grep Online: | awk -F : '{print $2}')
    hostname=$(sudo hostname)
else
    MASTER=$(sudo hostname)
    NODES=$(head -n 1 /home/akkadianuser/load_balancer.config)
    hostname=$(sudo hostname)
fi

ran=$(( ( RANDOM % 100 )  + 1 ))
gpg -d -q --batch --passphrase Akkadian /home/akkadianuser/.sshpassword_aco.gpg > /home/akkadianuser/.sshpass_$ran

date >> /home/akkadianuser/logs/set_mongo_replica.log
echo "master: $MASTER" >> /home/akkadianuser/logs/set_mongo_replica.log
echo "nodes: $NODES" >> /home/akkadianuser/logs/set_mongo_replica.log
echo "hostname: $hostname" >> /home/akkadianuser/logs/set_mongo_replica.log

# Get MASTER db and replicate to all nodes, to start all at same point
# checksum to see if the file is correctly passed into slaves.
# no matter who is master, just ensure db is same in all slaves.

sudo rm -Rf /opt/akkadian/data/restore/AcoServer
sudo rm -Rf acoServer_base.tar.gz
sudo mongodump -u mongoDBuser -p X46kTVmongo --authenticationDatabase admin --db AcoServer -o /opt/akkadian/data/restore/
sudo tar -zcvf /opt/akkadian/data/restore/acoServer_base.tar.gz -C /opt/akkadian/data/restore/AcoServer/ .

sudo systemctl stop aco-server
sudo systemctl stop tomcat

IFS=$' '
for j in $NODES
    do
        if [[ ( "$j" != "[" ) && ( "$j" != "]" ) && "$j" != "$hostname" ]]
        then
            sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo systemctl stop aco-server"
            sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo systemctl stop tomcat"
        fi
done

nodes_len=0
initialization_string=""
IFS=$' '
for j in $NODES
    do
        if [[ ( "$j" != "[" ) && ( "$j" != "]" ) ]];
        then
            nodes_len=$(($nodes_len+1))
            echo "processing start configs for: $j" >> /home/akkadianuser/logs/set_mongo_replica.log

            #Ensuring slaves have always same data
            if [[ "$j" != "$hostname" ]]
            then
                sshpass -f /home/akkadianuser/.sshpass_$ran scp -o StrictHostKeyChecking=no /opt/akkadian/data/restore/acoServer_base.tar.gz root@$j:/opt/akkadian/data/restore/acoServer_base.tar.gz
                sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo rm -Rf /opt/akkadian/data/restore/AcoServer"
                sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo mkdir /opt/akkadian/data/restore/AcoServer"
                sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "tar -zxvf /opt/akkadian/data/restore/acoServer_base.tar.gz -C /opt/akkadian/data/restore/AcoServer"
                sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sh /opt/akkadian/aco-server/scripts/mongoacorestore.sh"
                echo "Restoring db..." >> /home/akkadianuser/logs/set_mongo_replica.log
                sleep 15
            fi

            #sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo systemctl stop mongodb"
            sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo echo replicapass > /opt/akkadian/data/mongokey "
            sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo chmod 600 /opt/akkadian/data/mongokey "
            sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo cp -Rf /home/akkadianuser/scripts/mongo_replica_scripts/start-mongo-replica.sh /opt/akkadian/aco-server/scripts/start-mongodb.sh"
            sleep 15
            sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo systemctl stop mongodb"
            sleep 10

            if [[ -z $initialization_string ]];
            then
                echo "initialization_string empty"
                initialization_string=$(echo "$j")
            else
                initialization_string=$(echo "$initialization_string,$j")
                echo "initialization_string not empty"
            fi
        fi
done

sudo echo "the initialization string is: $initialization_string" >> /home/akkadianuser/logs/set_mongo_replica.log
sudo echo "$initialization_string" > /opt/akkadian/data/replica
sudo chmod 666 /opt/akkadian/data/replica
nodes_comma_separated=$(cat /opt/akkadian/data/replica)
yes | sudo cp -Rf /home/akkadianuser/scripts/mongo_replica_scripts/applicationContext-mongo_replicaSet.xml /tmp/applicationContext-mongo_replicaSet.xml
sudo sed -i "s/list_of_nodes/$nodes_comma_separated/g" /tmp/applicationContext-mongo_replicaSet.xml
yes | sudo cp -Rf /tmp/applicationContext-mongo_replicaSet.xml /opt/tomcat/webapps/acow-webapp/WEB-INF/applicationContext-mongo.xml
sleep 5

echo "initiate master node: $MASTER running " >> /home/akkadianuser/logs/set_mongo_replica.log
sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$MASTER 'date' >> /home/akkadianuser/mongo_replica.log
sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$MASTER "sudo systemctl start mongodb"
sleep 5
sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$MASTER 'mongo admin -u mongoDBuser -p X46kTVmongo --eval "rs.initiate()"' >> /home/akkadianuser/logs/set_mongo_replica.log
IFS=$' '
for j in $NODES
    do
        if [[ ( "$j" != "[" ) && ( "$j" != "]" ) ]]
        then
            echo "processing string for $j" >> /home/akkadianuser/logs/set_mongo_replica.log

            if [[ "$j" != "$hostname" ]]; then
                sshpass -f /home/akkadianuser/.sshpass_$ran scp -o StrictHostKeyChecking=no /opt/akkadian/data/replica root@$j:/opt/akkadian/data/replica
                sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo chmod 666 /opt/akkadian/data/replica"
                sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "yes | sudo cp -Rf /home/akkadianuser/scripts/mongo_replica_scripts/applicationContext-mongo_replicaSet.xml /tmp/applicationContext-mongo_replicaSet.xml"
                sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sed -i 's/list_of_nodes/$nodes_comma_separated/g' /tmp/applicationContext-mongo_replicaSet.xml"
                sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "yes | sudo cp -Rf  /tmp/applicationContext-mongo_replicaSet.xml /opt/tomcat/webapps/acow-webapp/WEB-INF/applicationContext-mongo.xml"
            fi

            tmp=$j
            if [[ "$MASTER" != "$j" ]]
            then
                sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo systemctl start mongodb"
                sleep 5
                echo "adding node: $j" >> /home/akkadianuser/logs/set_mongo_replica.log
                j="\"$j\""
                echo "adding node $j into the replica" >> /home/akkadianuser/logs/set_mongo_replica.log
                sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$MASTER "mongo admin -u mongoDBuser -p X46kTVmongo --eval 'rs.add($j)'"  >> /home/akkadianuser/mongo_replica.log
            fi
            echo "restarting aco-server in $j" >> /home/akkadianuser/logs/set_mongo_replica.log
            sleep 2
            sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$tmp "sudo echo '$MASTER' > /home/akkadianuser/master"
        fi
done

sleep 25

sudo systemctl start aco-server
sleep 5
sudo systemctl start tomcat

IFS=$' '
for j in $NODES
    do
        if [[ ( "$j" != "[" ) && ( "$j" != "]" ) && "$j" != "$hostname" ]]
        then
            sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo systemctl start aco-server"
            sleep 5
            sshpass -f /home/akkadianuser/.sshpass_$ran ssh -o StrictHostKeyChecking=no root@$j "sudo systemctl start tomcat"
        fi
done

sudo rm -Rf /home/akkadianuser/.sshpass_$ran

echo "ending replica configuration..." >> /home/akkadianuser/logs/set_mongo_replica.log
date >> /home/akkadianuser/logs/set_mongo_replica.log